// import 'package:cmsapp/utils/Rest.dart';
// import 'package:cmsapp/utils/StringUtil.dart';
// import 'package:hive/hive.dart';
import 'package:hive2moor/src/annotations.dart';

part 'StudentInfo.g.dart';

@HiveType(typeId: 0)
class StudentInfo {
  static StudentInfo _instance = StudentInfo._internal();

  StudentInfo._internal();

  //TODO Store whole objects from studentInfo instead of some values.
  factory StudentInfo() => _instance;

  @NullableMoor()
  @HiveField(0)
  String stuFirstName;
  @HiveField(1)
  String stuLastName;
  @HiveField(2)
  String stuMiddleName;
  @HiveField(3)
  String stuEmail;
  @HiveField(4)
  String stuMobile;
  @PK(ai: false)
  @HiveField(5)
  int stuId;
  @HiveField(6)
  String stuNationality;
  @HiveField(7)
  String stuCategory;
  @HiveField(8)
  String stuBloodgroup;
  @HiveField(9)
  String stuGender;
  @HiveField(10)
  String stuDob;
  @NullableMoor()
  @HiveField(11)
  String stuReligion;
  @HiveField(12)
  String stuPhoto;
  @HiveField(13)
  @HiveField(14)
  String stuPerAddress;
  @HiveField(15)
  String stuPerCity;
  @HiveField(16)
  String stuPerState;
  @HiveField(17)
  String stuPerCountry;
  @HiveField(18)
  String stuPerPincode;
  @HiveField(19)
  @HiveField(20)
  String stuCurrAddress;
  @HiveField(21)
  String stuCurrCity;
  @HiveField(22)
  String stuCurrState;
  @HiveField(23)
  String stuCurrCountry;
  @HiveField(24)
  String stuCurrPincode;

  @HiveField(25)
  String studentFatherFname;
  @HiveField(26)
  String studentFatherMname;
  @HiveField(27)
  String studentFatherLname;
  @HiveField(28)
  String studentFatherOccupation;
  @HiveField(29)
  String studentFatherContact;
  @HiveField(30)
  int studentFatherAnnualIncome;

  @HiveField(31)
  String studentMotherFname;
  @HiveField(32)
  String studentMotherMname;
  @HiveField(33)
  String studentMotherLname;

  @HiveField(34)
  int fcurrsem;
  @HiveField(35)
  int fCourseId;
  @HiveField(36)
  int fDeptId;
  @HiveField(37)
  int stuStatus;

  @HiveField(38)
  String batchName;
  @HiveField(39)
  int batchId;
  @HiveField(40)
  int classId;
  //Same as stuID
  @HiveField(41)
  int studentId;
  @HiveField(42)
  String deptName;
  @HiveField(43)
  String deptAlias;
  @HiveField(44)
  String deptCode;

  @HiveField(45)
  String stuEnroll;
  @HiveField(46)
  bool boolEan;

  factory StudentInfo.fromJson(Map<String, dynamic> parsedJson) {
    return StudentInfo()
      ..stuFirstName = parsedJson["stuFirstname"]
      ..stuMiddleName = parsedJson["stuMiddlename"]
      ..stuLastName = parsedJson["stuLastname"]
      ..stuGender = parsedJson["stuGender"] ?? "N/A"
      ..stuId = parsedJson["stuId"] ?? -1
      ..stuPhoto = parsedJson["stuPhoto"]
      ..stuReligion = parsedJson["stuReligion"] ?? "N/A"
      ..stuDob = parsedJson["stuDob"] ?? "N/A"
      ..stuNationality = parsedJson["stuNationality"] ?? "N/A"
      ..stuBloodgroup = parsedJson["stuBloodgroup"] ?? "N/A"
      ..stuPerAddress = parsedJson["stuPerAddress"] ?? "N/A"
      ..stuPerCity = parsedJson["stuPerCity"] ?? "N/A"
      ..stuPerState = parsedJson["stuPerState"] ?? "N/A"
      ..stuPerCountry = parsedJson["stuPerCountry"] ?? "N/A"
      ..stuPerPincode = parsedJson["stuPerPincode"] ?? "N/A"
      ..stuCurrAddress = parsedJson["stuCurrAddress"] ?? "N/A"
      ..stuCurrCity = parsedJson["stuCurrCity"] ?? "N/A"
      ..stuCurrState = parsedJson["stuCurrState"] ?? "N/A"
      ..stuCurrCountry = parsedJson["stuCurrCountry"] ?? "N/A"
      ..stuCurrPincode = parsedJson["stuCurrPincode"] ?? "N/A"
      ..stuEmail = parsedJson["stuEmail"] ?? "N/A"
      ..stuMobile = parsedJson["stuMobile"] ?? "N/A"
      ..studentFatherFname = parsedJson["studentFatherFname"]
      ..studentFatherMname = parsedJson["studentFatherMname"]
      ..studentFatherLname = parsedJson["studentFatherLname"]
      ..studentFatherOccupation = parsedJson["studentFatherOccupation"] ?? "N/A"
      ..studentFatherContact = parsedJson["studentFatherContact"] ?? "N/A"
      ..studentFatherAnnualIncome =
          parsedJson["studentFatherAnnualIncome"] ?? -1
      ..studentMotherFname = parsedJson["studentMotherFname"]
      ..studentMotherMname = parsedJson["studentMotherMname"]
      ..studentMotherLname = parsedJson["studentMotherLname"]
      ..stuCategory = parsedJson["stuCategory"] ?? "N/A"
      ..fcurrsem = parsedJson["fcurrsem"] ?? -1
      ..fCourseId = parsedJson["fCourseId"] ?? -1
      ..fDeptId = parsedJson["fDeptId"] ?? -1
      ..stuStatus = parsedJson["stuStatus"] ?? -1;
  }
  Future<StudentInfo> init({bool disableHive = false}) async {
    if (disableHive) {
      return await Rest().loadStudentInfo(stuEnroll);
    }
    var stuBox = Hive.box('StudentInfo');
    if (stuBox.isEmpty) {
      print(stuEnroll);
      var stuInfoRest = await Rest().loadStudentInfo(stuEnroll);
//      stuInfoRest..stuEnroll = enroll;
      stuBox.add(stuInfoRest);
      return stuInfoRest;
    }
    print("Fetching from Database");
    return stuBox.get(0);
  }

  String _getFullName(String fName, String mName, String lName) {
    if (fName == null) {
      return "N/A";
    }
    mName = mName != null ? mName.trim() + " " : "";
    lName = lName ?? "";
    return fName.trim() + " " + mName + lName.trim();
  }

  String getStudentFullName() {
    return _getFullName(StringUtil.capitalise(stuFirstName), null,
        StringUtil.capitalise(stuLastName));
  }

  String getDeptName() {
    String fName = StringUtil.capitalise(deptName.split(" ")[0]);
    String lName = StringUtil.capitalise(deptName.split(" ")[1]);
    if (fName != null && lName != null) {
      return fName + " " + lName;
    }
    return null;
  }

  String getSmolBatchName() {
    if (batchName == null) {
      return null;
    }
    String year = batchName.split(" ")[0];
    String section = batchName.split(" ")[3];
    String batch = batchName.split(" ")[4];
    return year + " " + section + " - " + batch;
  }

  String getFathersFullName() {
    return _getFullName(
        StringUtil.capitalise(studentFatherFname),
        StringUtil.capitalise(studentFatherMname),
        StringUtil.capitalise(studentFatherLname));
  }

  String getMothersFullName() {
    return _getFullName(
        StringUtil.capitalise(studentMotherFname),
        StringUtil.capitalise(studentMotherMname),
        StringUtil.capitalise(studentMotherLname));
  }
}
