import 'package:hive2moor/src/annotations.dart';

part 'TimeTableRecordInfo.g.dart';

@HiveType(typeId: 3)
class TimeTableRecordInfo {
  @HiveField(0)
  int ttId;
  @HiveField(1)
  int fTimetableId;
  @HiveField(2)
  int fSubjectId;
  @HiveField(3)
  int fFacultyId;
  @HiveField(4)
  int fLocationId;
  @NullableMoor()
  @HiveField(5)
  int fDeptId;
  @HiveField(6)
  int ttSem;
  @HiveField(7)
  int fCourseId;
  @HiveField(8)
  int fClassId;
  @HiveField(9)
  int fBatchId;
  @HiveField(10)
  String ttDay;
  @HiveField(11)
  String ttStartTime;
  @HiveField(12)
  String ttEndTime;
  @HiveField(13)
  double ttDuration;
  @HiveField(14)
  String ttLoadType;
  @HiveField(15)
  bool isFree;

  @HiveField(16)
  SubjectInfo subInfo;
  @HiveField(17)
  LocationInfo locInfo;
  @HiveField(18)
  FacultyInfo facInfo;

  @HiveField(19)
  double
      startTimeDouble; // this is calculated by dividing the hour by 24 and minutes by (24*60) and so on...
  @HiveField(20)
  double durationDouble;

  TimeTableRecordInfo(
      {this.ttId,
      this.fTimetableId,
      this.fSubjectId,
      this.fFacultyId,
      this.fLocationId,
      this.fDeptId,
      this.ttSem,
      this.fCourseId,
      this.fClassId,
      this.fBatchId,
      this.ttDay,
      this.ttStartTime,
      this.ttEndTime,
      this.ttDuration,
      this.ttLoadType,
      // this.subInfo,
      // this.locInfo,
      this.isFree,
      this.startTimeDouble,
      this.durationDouble});

  factory TimeTableRecordInfo.fromJson(Map<String, dynamic> parsedJson) {
    return TimeTableRecordInfo(
        ttId: parsedJson["ttId"],
        fTimetableId: parsedJson["fTimetableId"],
        fSubjectId: parsedJson["fSubjectId"],
        fFacultyId: parsedJson["fFacultyId"],
        fLocationId: parsedJson["fLocationId"],
        fDeptId: parsedJson["fDeptId"],
        ttSem: parsedJson["ttSem"],
        fCourseId: parsedJson["fCourseId"],
        fClassId: parsedJson["fClassId"],
        fBatchId: parsedJson["fBatchId"],
        ttDay: parsedJson["ttDay"],
        ttStartTime: DateUtil.convertTo24(parsedJson["ttStartTime"]),
        ttEndTime: DateUtil.convertTo24(parsedJson["ttEndTime"]),
        ttDuration: double.parse(parsedJson["ttDuration"].toString()),
        ttLoadType: parsedJson["ttLoadType"],
        isFree: false,
        startTimeDouble:
            getExcelDouble(parsedJson["ttStartTime"], is24Hour: false),
        durationDouble:
            getExcelDouble(parsedJson["ttEndTime"], is24Hour: false) -
                getExcelDouble(parsedJson["ttStartTime"], is24Hour: false));
  }

  factory TimeTableRecordInfo.getFree(
      String startTime, String endTime, String day) {
    return TimeTableRecordInfo(
      isFree: true,
      subInfo: SubjectInfo.getFree(),
      ttStartTime: startTime,
      ttEndTime: endTime,
      ttDay: day,
      startTimeDouble: getExcelDouble(startTime),
      ttDuration: double.parse(endTime.split(":")[0]) -
          double.parse(startTime.split(":")[0]),
      durationDouble: getExcelDouble(endTime) - getExcelDouble(startTime),
    );
  }

  factory TimeTableRecordInfo.getCollageStart(String day) {
    return TimeTableRecordInfo(
        ttStartTime: "9:45:00",
        ttEndTime: "9:45:00",
        ttDay: day,
        ttDuration: 0,
        subInfo: SubjectInfo(
          subName: "CollageStartTime",
          subAlias: "CST",
        ),
        startTimeDouble: getExcelDouble("9:45:00"),
        durationDouble: 0,
        isFree: true);
  }

  factory TimeTableRecordInfo.getCollageEnd(String day) {
    return TimeTableRecordInfo(
        ttStartTime: "16:30:00",
        ttEndTime: "16:30:00",
        ttDay: day,
        ttDuration: 0,
        subInfo: SubjectInfo(
          subName: "CollageEndTime",
          subAlias: "CET",
        ),
        startTimeDouble: getExcelDouble("16:30:00"),
        durationDouble: 0,
        isFree: true);
  }

  factory TimeTableRecordInfo.getRecess(
      String day, String startTime, String endTime, double startTimeDouble) {
    return TimeTableRecordInfo(
        ttStartTime: startTime,
        ttEndTime: endTime,
        ttDay: day,
        //TODO: ttDuration for recess and also for every other factory mehthod
        ttDuration: 0,
        subInfo: SubjectInfo(
          subName: "Recess",
          subAlias: "Recess",
        ),
        startTimeDouble: startTimeDouble,
        durationDouble: getExcelDouble(endTime) - getExcelDouble(startTime),
        isFree: true);
  }

  static double getExcelDouble(String time, {bool is24Hour = true}) {
    if (!is24Hour) {
      String hour = DateUtil.convertTo24(time).split(":")[0];
      String minute = DateUtil.convertTo24(time).split(":")[1];

      return (double.parse(hour) / 24) + (double.parse(minute) / (60 * 24));
    }
    return double.parse(time.split(":")[0]) / 24 +
        double.parse(time.split(":")[1]) / (24 * 60);
  }
}
